/*
 * YDefaultVisualContainer.java
 *
 * Created on 17. April 2000, 22:49
 */
 
package YARB.Main;
import YARB.Interfaces.*;
import YARB.Dialogs.*;
import java.util.ArrayList;
import javax.swing.*;

/** 
 * The default container for visualization configuration
 * @author  Thomas
 * @version 
 */
public class YDefaultVisualContainer 
extends Object 
implements IVisualContainer
{

  protected ArrayList   m_Filter;
  protected IFileSource m_File;
  protected IDataViewer m_Viewer;
  protected String      m_strFile;
  
  /** Creates new YDefaultVisualContainer */
  public YDefaultVisualContainer() 
  {
    m_Filter=new ArrayList();
  }
  
  public void setFileSource(IFileSource FS) 
  {
    m_File=FS;
  }
  
  public void setDataView(IDataViewer DV) 
  {
    m_Viewer=DV;
  }
  
  public int numbOfDataFilter() 
  {
    return m_Filter.size();
  }
  
  public void setDataFilter(int nPos,IDataFilter DF) 
  {
    m_Filter.add(nPos,DF);
  }
  
  public void addDataFilter(IDataFilter DF) 
  {
    m_Filter.add(DF);
  }

  public void deleteDataFilter(int nPos) 
  {
    m_Filter.remove(nPos);
  }
  
 public boolean SaveConfiguration(IConfigurationWriter Cfg) throws IConfigurationWriter.ECfgWriterError
  {
    if(Cfg==null){
      return false;
    };
    
    // open general cfg
    Cfg.CreateNode("YARBConfiguration");
    // My own properties:
    Cfg.addTitle(getName());
    Cfg.addInt(m_Filter.size());  // number of filters
    if(m_strFile!=null)
    {
      Cfg.addFile(m_strFile);
    }
    else 
    {
      Cfg.addFile("none");
    };
    
    // File source:
    Cfg.CreateNode("FileSources");
    
    if(m_File!=null)
    { 
      Cfg.addClass(cutClassName(m_File.toString()));
      m_File.SaveConfiguration(Cfg);
    }
    else
    {
      Cfg.addClass("_none_");
    };
    Cfg.CloseNode("FileSources");
    
    // Filters:
    for(int i=0; i<m_Filter.size();i++)
    {
      Cfg.CreateNode("DataFilter");

      IDataFilter df=(IDataFilter)m_Filter.get(i);
      Cfg.addClass(cutClassName(df.toString()));
      df.SaveConfiguration(Cfg);

      Cfg.CloseNode("DataFilter");
    };
    

    // Viewer:
    Cfg.CreateNode("DataViewer");
    
    if(m_Viewer!=null)
    { 
      Cfg.addClass(cutClassName(m_Viewer.toString()));
      m_Viewer.SaveConfiguration(Cfg);
    }
    else
    {
      Cfg.addClass("_none_");
    };
    Cfg.CloseNode("DataViewer");
    
    
    // Close all cfg
    Cfg.CloseNode("YARBConfiguration");
    
    return true;
  }
  
 private String cutClassName(String str)
  {
    int i= str.lastIndexOf('@');
    if(i==-1)
    {
      return str;
    };
    
    return str.substring(0,i);
  }
  
 public boolean LoadConfiguration(IConfigurationReader Cfg) throws IConfigurationReader.ECfgReaderError
  {
    if(Cfg==null){
      return false;
    };
    
    String str;
    // open main cfg
    Cfg.OpenNode("YARBConfiguration");
    
    // get my own properties:
    setName(Cfg.getTitle());
    int nFilters=Cfg.getInt();
    m_strFile=Cfg.getFile();
    if(m_strFile.equals("none")) m_strFile=null;
    // File source:
    Cfg.OpenNode("FileSources");
    str=Cfg.GetClass();
    if(!str.equals("none"))
    { 
      IFileSource fs=(IFileSource) createClassFromNameAndCheckInterface(str, "YARB.Interfaces.IFileSource");
      setFileSource(fs);
      fs.LoadConfiguration(Cfg);
    }

    // Filters:
    for(int i=0; i<nFilters;i++)
    {
      Cfg.OpenNode("DataFilter");
      str=Cfg.GetClass();
      IDataFilter df=(IDataFilter) createClassFromNameAndCheckInterface(str, "YARB.Interfaces.IDataFilter");
      addDataFilter(df);
      df.LoadConfiguration(Cfg);
    };
    
    // File source:
    Cfg.OpenNode("DataViewer");
    str=Cfg.GetClass();
    if(!str.equals("none"))
    { 
      IDataViewer dv=(IDataViewer) createClassFromNameAndCheckInterface(str, "YARB.Interfaces.IDataViewer");
      setDataView(dv);
      dv.LoadConfiguration(Cfg);
    }

    //everything wnet O.K.
    return true;
  }
  
  private Object createClassFromNameAndCheckInterface(String className, String interfaceName) throws IConfigurationReader.ECfgReaderError
  {
    try {
            // load the class and search for the required interface
            Class classDefinition = Class.forName(className);
            Class[] theInterfaces = classDefinition.getInterfaces();
            for (int i = 0; i < theInterfaces.length; i++) 
            {
              String str=theInterfaces[i].getName();
               // interface found? then create a new instance, add the container to our list and return it.
               if( str.equals(interfaceName))
               {
                  Object o= classDefinition.newInstance();
                 return o;                  
                };
            }          
          
            // oops. interface not found!
            System.out.println("Error: Loaded class " + className + " didn't implement required"  + interfaceName);
            throw new IConfigurationReader.ECfgReaderError("Error: Loaded class " + className + " didn't implement required interface"  + interfaceName);
          } 
          catch (InstantiationException e) {
            System.out.println(e);    
            throw new IConfigurationReader.ECfgReaderError("Error creating class " + className + ".\nERROR: " + e.toString());
          } 
          catch (IllegalAccessException e) {
            System.out.println(e);
            throw new IConfigurationReader.ECfgReaderError("Error creating class " + className + ".\nERROR: " + e.toString());
          } 
          catch (ClassNotFoundException e) {
            System.out.println(e);
            throw new IConfigurationReader.ECfgReaderError("Error creating class " + className + ".\nERROR: " + e.toString());
          }
  }
  
  
  public boolean IsConfigurable() 
  {
    return true;
  }
  
  public boolean ShowConfigurationDialog(JFrame ParentFrame) 
  {
    YStandardNameChooser dlg= new YStandardNameChooser(ParentFrame, "YARB Configuration Properties",true,getName());
    dlg.show();
    
    setName(dlg.getName());
    return true;  
  }

  public boolean DoVisualization(IMainFrame ParentFrame) throws IDataProvider.DataProcessingError
  {
    if (m_File==null)
    {
      return false;
    };
    
    m_File.setFileName(m_strFile);
    
    if(m_Viewer==null)
    {
      return false;
    };
    
    m_Viewer.setFileName(m_strFile);
    
    if(m_Filter.size() > 0)
    {
      int i=m_Filter.size()-1;

      m_Viewer.setDataSource((IDataFilter)m_Filter.get(i));
      
      while(i>0)
      {
        ((IDataFilter)m_Filter.get(i)).setPreviousProvider((IDataFilter)m_Filter.get(i-1));
        i--;
      };
      
      ((IDataFilter)m_Filter.get(0)).setPreviousProvider(m_File);    
    }
    else
    {
      m_Viewer.setDataSource(m_File);
    };
  
    m_Viewer.setCaption(getName());
    
    return m_Viewer.DoVisualize(ParentFrame);
 
  }
  public void setFile(String strFile)
  {
    m_strFile=strFile;
  }
  
  
  /**
  * Contains the assigned name string.
  */
  private String  m_strName;
  
  /**
  * Assigns a name to the current object.
  */
  public void setName(String strName){
    m_strName=strName;
  }
    
  /**
  * Returns the assigned name of the object
  */
  public String  getName(){
    return m_strName;
  }

  public IFileSource getFileSource() {
    return m_File;
  }
  public IDataViewer getDataView() {
    return m_Viewer;
  }
  public IDataFilter getDataFilter(int nPos) {
    return (IDataFilter) m_Filter.get(nPos);
  }
 
}
