/*
 * YXMLWriter.java
 *
 * Created on 4. Juni 2000, 20:42
 */
 
package YARB.XML;

import java.io.*;


/** 
 *
 * @author  Administrator
 * @version 
 */
public class YXMLWriter extends Object {
  // Source file
  private String m_XMLSourceFile = null;
  // Tree with XML data
  private String[] m_nodeTree = null;
  private int m_InodeTreeLength = 0;
  // Tree for tracking nodes
  private String[] m_TrackOpenNodes = null;
  private int m_ITrackOpenNodesLength = 0;
  
  /** Creates new YXMLWriter */
  public YXMLWriter() {
  }

  /** Set XML Source File */
  public void SetXMLSource(String src) {
    m_XMLSourceFile = src;
  }
  // Valid Elements in XML File
  static final String[] m_validElements = {
    //"YARBMultipleConfiguration", "YARBSingleConfiguration",
    "YARBConfiguration",
    "DataFilter", "FileSources", "DataViewer",
    "Title", "Class", "File",
    "boolean", "int", "float", "String",
  };
  // Amount of valid elements in XML File
  static final int m_IvalidElements = 11;

  public void ResetAll() {
    // Source file
    m_XMLSourceFile = null;
    // Tree with XML data
    m_nodeTree = null;
    m_InodeTreeLength = 0;
    // Tree for tracking nodes
    m_TrackOpenNodes = null;
    int m_ITrackOpenNodesLength = 0;    
  }
  
  private boolean isElementValid(String ElementName) {
    for( int i=0; i < m_IvalidElements; i++) {
      if( ElementName.compareTo(m_validElements[i]) == 0 ) return true;
    }
    return false;
  }
  
  public void printNodeTree() {    
    for( int i=0; i < m_InodeTreeLength; i++) System.out.println(m_nodeTree[i]);
  }
  public void printTrack() {
    for( int i=0; i < m_ITrackOpenNodesLength; i++) System.out.println(m_TrackOpenNodes[i]);    
  }

  /** Write XML data out to a file */
  
  public void writeXMLFile() throws ECgfWriterCGeneralError {    
    if( m_ITrackOpenNodesLength != 0 ) {
      throw new ECgfWriterCGeneralError("Can't create XML file, not all nodes have been closed ! " + this.toString());
    }
    try {
      BufferedWriter OutputSrc = new BufferedWriter( new FileWriter(m_XMLSourceFile) );
      
      // Header
      OutputSrc.write("<?xml version='1.0' encoding='UTF-8'?>");
      OutputSrc.newLine();
      OutputSrc.write("<!DOCTYPE YARBConfigurationDTD SYSTEM \"YARBConfiguration.dtd\" [] >");
      OutputSrc.newLine();
      // NodeTree
      for( int i=0; i < m_InodeTreeLength; i++) {
        OutputSrc.write(m_nodeTree[i]);
        if( m_nodeTree[i].startsWith("</") ) OutputSrc.newLine();
        else {
          if( m_nodeTree[i].charAt(0) == '<' && m_nodeTree[i+1].charAt(0) == '<' ) OutputSrc.newLine();
        }
        
      }
      
      OutputSrc.close();
    } catch (IOException e) {
      System.out.println("***IOException: " + e.toString());
    }          
  
  }
  
  /** Add node to tree */
  public void addNode(String NodeName) throws ECgfWriterCGeneralError {
    if( NodeName == null ) {
      throw new ECgfWriterCGeneralError("Null pointer occured ! " + this.toString() );
    }
    if( !isElementValid(NodeName) ) {
      throw new ECgfWriterCGeneralError(NodeName + " isn't a valid node for YARB! " + this.toString() );
    }
    // format string to <NodeName>    
    StringBuffer tmp = new StringBuffer("<");
    tmp.insert(tmp.length(), NodeName);
    tmp.append(">");
    // add to NodeTree
    if( m_InodeTreeLength == 0 ) {
      // first node
      if( NodeName.compareTo("YARBConfiguration") != 0 )
        throw new ECgfWriterCGeneralError("First node in Tree must be <YARBConfiguration> ! "+this.toString());
      m_InodeTreeLength++;
      m_ITrackOpenNodesLength++;      
      m_nodeTree = new String[1];
      m_nodeTree[0] = tmp.toString();
      m_TrackOpenNodes = new String[1];
      m_TrackOpenNodes[0] = NodeName;
      return ;
    }
    // Save old nodeTree
    String[] tmpTree = m_nodeTree;
    // Create new nodeTree
    m_InodeTreeLength++;
    m_nodeTree = new String[m_InodeTreeLength];
    for( int i=0; i < m_InodeTreeLength-1; i++)
      m_nodeTree[i] = tmpTree[i];         
    m_nodeTree[m_InodeTreeLength-1] = tmp.toString();
        
    // Save old tracking
    tmpTree = m_TrackOpenNodes;
    // Create new nodeTree
    m_ITrackOpenNodesLength++;
    m_TrackOpenNodes = new String[m_ITrackOpenNodesLength];
    for( int i=0; i < m_ITrackOpenNodesLength-1; i++)
      m_TrackOpenNodes[i] = tmpTree[i];         
    m_TrackOpenNodes[m_ITrackOpenNodesLength-1] = NodeName;
    
  }
  
  /** Close node from tree */
  public void closeNode(String NodeName) throws ECgfWriterCGeneralError {
    if( NodeName == null ) {
      throw new ECgfWriterCGeneralError("Null pointer occured ! " + this.toString() );
    }    
    if( !isElementValid(NodeName) ) {
      throw new ECgfWriterCGeneralError(NodeName + " isn't a valid node for YARB! " + this.toString() );
    }
      
    if( m_ITrackOpenNodesLength == 0 ) {
      throw new ECgfWriterCGeneralError("No node is open to close ! " + this.toString());
    }
    // node to close must be last in list
    if( NodeName.compareTo(m_TrackOpenNodes[m_ITrackOpenNodesLength-1]) != 0 ) {
      throw new ECgfWriterCGeneralError("Before node <"+NodeName+"> other nodes have to be closed first ! (last = " + m_TrackOpenNodes[m_ITrackOpenNodesLength-1] + ")" + this.toString());
    }
    // Delete last element
    m_ITrackOpenNodesLength--;
    String[] tmpTree = m_TrackOpenNodes;
    m_TrackOpenNodes = new String[m_ITrackOpenNodesLength];
    for( int i=0; i < m_ITrackOpenNodesLength; i++) m_TrackOpenNodes[i] = tmpTree[i];
    // Add to NodeTree

    // format string to </NodeName>    
    StringBuffer tmp = new StringBuffer("</");
    tmp.insert(tmp.length(), NodeName);
    tmp.append(">");

    // Save old nodeTree
    tmpTree = m_nodeTree;
    // Create new nodeTree
    m_InodeTreeLength++;
    m_nodeTree = new String[m_InodeTreeLength];
    for( int i=0; i < m_InodeTreeLength-1; i++)
      m_nodeTree[i] = tmpTree[i];         
    m_nodeTree[m_InodeTreeLength-1] = tmp.toString();
    
    // m_ITrackOpenNodesLength triggers write out if 0
    /*if(m_ITrackOpenNodesLength == 0) {
      writeXMLFile();
    }*/
    
  }
    
  /** Add element to tree */
  public void addElement(String ElementName, String ElementValue) throws ECgfWriterCGeneralError {
    if( ElementName == null ) {
      throw new ECgfWriterCGeneralError("Null pointer occured ! " + this.toString() );
    }
        
    if( !isElementValid(ElementName) ) {
      throw new ECgfWriterCGeneralError(ElementName + " isn't a valid node for YARB! " + this.toString() );
    }      
    try {
      if( ElementName != null ) addNode(ElementName);
      else addNode("none");
      // add value to tree
      // Save old nodeTree
      String [] tmpTree = m_nodeTree;
      // Create new nodeTree
      m_InodeTreeLength++;
      m_nodeTree = new String[m_InodeTreeLength];
      for( int i=0; i < m_InodeTreeLength-1; i++)
        m_nodeTree[i] = tmpTree[i];         
      m_nodeTree[m_InodeTreeLength-1] = ElementValue;
      
      closeNode(ElementName);
    } catch(ECgfWriterCGeneralError e) {
      throw new ECgfWriterCGeneralError( e.toString() + " " + this.toString() );
    }
  }  
  /*
  public static void main (String args[]) {
    YXMLWriter test = new YXMLWriter();
    test.SetXMLSource("e:\\temp\\test.xml");
    try {
      test.addNode("YARBConfiguration");
      test.addElement("Title","YARB");
      test.addNode("FileSources");
      test.addElement("Class",test.toString());
      test.addElement("Title","Pattern");
      test.addElement("File","d:\\examples\\xor.pat");
      test.addElement("Class","YARB.FileSources.YFSSNNSPatternFiles");
      test.addElement("boolean","true");
      test.addElement("int","12345");
      test.closeNode("FileSources");
      test.closeNode("YARBConfiguration");
      //test.printNodeTree();
      //test.printTrack();
      test.writeXMLFile();
      
    } catch(ECgfWriterCGeneralError e) {
      System.out.println(e.toString() );
    }
  }
  */
  /**
  * general error exception class for this interface
  */
  class ECgfWriterCGeneralError extends Exception
  {
    ECgfWriterCGeneralError(String description)
    {
      super(description);
    }
  }  
  
}